' File: CLinkedList.inc
'
type p_listnode
	pointer _data
	pointer _next
	pointer prev
end type

class CLinkedList
Private
	pointer head
	pointer tail
public
	declare CLinkedList( )
	declare _CLinkedList( )

	declare add( pointer d ), pointer
	declare addHead( pointer d ), pointer
	declare insertAfter( pointer node, pointer d ), pointer
	declare insertBefore( pointer node, pointer d ), pointer

	declare remove( pointer node ), pointer
	declare removeAll( )

	declare getFirst( ), pointer
	declare getLast( ), pointer
	declare getNext( pointer node ), pointer
	declare getPrev( pointer node ), pointer

	declare nodeData( pointer node ), pointer

	declare enumerate( uint cb )
	declare enumerateBackwards( uint cb )
	declare SearchFor( uint cb ), int
end class

declare __listcb( pointer d )
declare __srchcb( pointer d, int eol ), int

sub llfree( pointer _data )
	delete _data
end sub

sub CLinkedList::CLinkedList( )
	head = new( p_listnode, 1 )
	tail = new( p_listnode, 1 )

	*<p_listnode>head._next = tail
	*<p_listnode>head._data = 0
	*<p_listnode>head.prev = 0

	*<p_listnode>tail._next = 0
	*<p_listnode>tail._data = 0
	*<p_listnode>tail.prev = head
end sub

sub CLinkedList::_CLinkedList( )
	removeAll( )
	delete head
	delete tail
end sub

sub CLinkedList::add( pointer d )
	return insertBefore( tail, d )
end sub

sub CLinkedList::addHead( pointer d )
	return insertAfter( head, d )
end sub

sub CLinkedList::insertAfter( pointer node, pointer d )
	pointer newnode : newnode = new( p_listnode, 1 )

	*<p_listnode>newnode.prev = node
	*<p_listnode>newnode._next = *<p_listnode>node._next
	if *<p_listnode>node._next then *<p_listnode>node.*<p_listnode>_next.prev = newnode
	*<p_listnode>node._next = newnode

	*<p_listnode>newnode._data = d

	return d
end sub

sub CLinkedList::insertBefore( pointer node, pointer d )
	pointer newnode : newnode = new( p_listnode, 1 )

	*<p_listnode>newnode._next = node
	*<p_listnode>newnode.prev = *<p_listnode>node.prev
	if *<p_listnode>node.prev <> 0 then *<p_listnode>node.*<p_listnode>prev._next = newnode
	*<p_listnode>node.prev = newnode

	*<p_listnode>newnode._data = d

	return d
end sub

sub CLinkedList::remove( pointer node )
	'' Can't remove head or tail
	if *<p_listnode>node._next = 0 or *<p_listnode>node.prev = 0 then return 0

	pointer d : d = *<p_listnode>node._data

	delete node

	*<p_listnode>node.*<p_listnode>_next.prev = *<p_listnode>node.prev
	*<p_listnode>node.*<p_listnode>prev._next = *<p_listnode>node._next

	return d
end sub

sub CLinkedList::removeAll( )
	pointer node : node = *<p_listnode>head._next
	pointer temp

	while node <> tail
		temp = *<p_listnode>node._next
		delete node
		node = temp
	wend
end sub

sub CLinkedList::getFirst( )
	return *<p_listnode>head._next
end sub

sub CLinkedList::getLast( )
	return *<p_listnode>tail.prev
end sub

sub CLinkedList::getNext( pointer node )
	pointer nxt : nxt = *<p_listnode>node._next
	if nxt <> tail then return nxt
	return 0
end sub

sub CLinkedList::getPrev( pointer node )
	pointer prv : prv = *<p_listnode>node.prev
	if prv <> head then return prv
	return 0
end sub

sub CLinkedList::nodeData( pointer node )
	return *<p_listnode>node._data
end sub

sub CLinkedList::enumerate( uint cb )
	pointer node
	node = getFirst( )
	while node <> 0
		!<__listcb>cb( *<p_listnode>node._data )
		node = getNext( node )
	wend
end sub

sub CLinkedList::enumerateBackwards( uint cb )
	pointer node
	node = getLast( )
	while node <> 0
		!<__listcb>cb( *<p_listnode>node._data )
		node = getPrev( node )
	wend
end sub
'
